@@ -1,21 +1,22 @@
-#' Explain details of a tbl.
+#' Explain details of a tbl
 #'
 #' This is a generic function which gives more details about an object than
-#' \code{\link{print}}, and is more focussed on human readable output than
-#' \code{\link{str}}.
+#' [print()], and is more focussed on human readable output than
+#' [str()].
 #'
 #' @section Databases:
-#' Explaining a \code{tbl_sql} will run the SQL \code{EXPLAIN} command which
+#' Explaining a `tbl_sql` will run the SQL `EXPLAIN` command which
 #' will describe the query plan. This requires a little bit of knowledge about
-#' how \code{EXPLAIN} works for your database, but is very useful for
+#' how `EXPLAIN` works for your database, but is very useful for
 #' diagnosing performance problems.
 #'
 #' @export
 #' @param x An object to explain
 #' @param ... Other parameters possibly used by generic
+#' @return The first argument, invisibly.
 #' @examples
 #' \donttest{
-#' if (require("RSQLite") && has_lahman("sqlite")) {
+#' if (require("dbplyr")) {
 #'
 #' lahman_s <- lahman_sqlite()
 #' batting <- tbl(lahman_s, "Batting")
@@ -38,18 +39,8 @@ explain <- function(x, ...) {
   UseMethod("explain")
 }
 
-#' @export
-explain.tbl_sql <- function(x, ...) {
-  force(x)
-  show_query(x)
-  message("\n")
-  message("<PLAN>\n", db_explain(x$src$con, sql_render(x)))
-
-  invisible(NULL)
-}
-
 #' @export
 #' @rdname explain
-show_query <- function(x) {
-  message("<SQL>\n", sql_render(x))
+show_query <- function(x, ...) {
+  UseMethod("show_query")
 }
