@@ -1,27 +1,37 @@
+Rcpp_version <- utils::packageVersion("Rcpp")
+
+R_version <- R.version.string
+
 .onLoad <- function(libname, pkgname) {
+  if (Rcpp_version != utils::packageVersion("Rcpp")) {
+    warning(
+      "Installed Rcpp (", utils::packageVersion("Rcpp"), ") different from ",
+      "Rcpp used to build dplyr (", Rcpp_version, ").\n",
+      "Please reinstall dplyr to avoid random crashes or undefined behavior.",
+      call. = FALSE
+    )
+  }
+
+  # if (R_version != R.version.string) {
+  #   warning(
+  #     "Installed R (", R.version.string, ") different from ",
+  #     "R used to build dplyr (", R_version, ").\n",
+  #     "Please reinstall dplyr to avoid random crashes or undefined behavior.",
+  #     call. = FALSE
+  #   )
+  # }
+
   op <- options()
   op.dplyr <- list(
-    dplyr.strict_sql = FALSE,
     dplyr.show_progress = TRUE
   )
   toset <- !(names(op.dplyr) %in% names(op))
-  if(any(toset)) options(op.dplyr[toset])
+  if (any(toset)) options(op.dplyr[toset])
 
   invisible()
 }
 
 .onAttach <- function(libname, pkgname) {
-  when_attached("data.table", {
-    if (!is_attached("dtplyr")) {
-      packageStartupMessage(rule())
-      packageStartupMessage(
-        "data.table + dplyr code now lives in dtplyr.\n",
-        "Please library(dtplyr)!"
-      )
-      packageStartupMessage(rule())
-    }
-  })
-
   setHook(packageEvent("plyr", "attach"), function(...) {
     packageStartupMessage(rule())
     packageStartupMessage("You have loaded plyr after dplyr - this is likely ",
